// Marker transparency
@cluster-outer-transparency: 70%;
@cluster-inner-transparency: 47%;

[class^="icon-"]:before, [class*=" icon-"]:before {
  margin-right: 0;
}

.awesome-marker i {
  margin-top: 8px;
}

.map {
  width: 100%;
  height: 100%;
  z-index: 0;
}

.content {
  height: 100%;
  padding: 0 !important;
}

.scroll-view {
  max-height: 50vh;
  height: 100%;
  overflow-y: auto;
}

.scroll-view table {
  white-space: nowrap;
  border-collapse: separate;
  border-spacing: 0 1px;
}

.map-popup-services {
  overflow: hidden;
}

#hoststatus {
  color: white;
  text-align: center;
  font-weight: bold;
  background-color: @color-down;
  padding: 0.25em;
  text-transform: uppercase;
}

.marker-cluster span {
  font-weight: normal;
  mix-blend-mode: difference;
}

// cluster OK
.marker-cluster-0 {
  color: @color-ok;
  background-color: fade(@color-ok, @cluster-inner-transparency);
}

.marker-cluster-0 div {
  background-color: fade(@color-ok, @cluster-outer-transparency);
}

.marker-cluster-0 span {
  color: @color-ok;
}

// cluster WARNING
.marker-cluster-1 {
  background-color: fade(@color-warning, @cluster-inner-transparency);
}

.marker-cluster-1 div {
  background-color: fade(@color-warning, @cluster-outer-transparency);
}

.marker-cluster-1 span {
  color: @color-warning;
}

// cluster CRITICAL
.marker-cluster-2 {
  background-color: fade(@color-critical, @cluster-inner-transparency);
}

.marker-cluster-2 div {
  background-color: fade(@color-critical, @cluster-outer-transparency);
}

.marker-cluster-2 span {
  color: @color-critical;
}

// cluster UNKNOWN
.marker-cluster-3 {
  background-color: fade(@color-unknown, @cluster-inner-transparency);
}

.marker-cluster-3 div {
  background-color: fade(@color-unknown, @cluster-outer-transparency);
}

.marker-cluster-3 span {
  color: @color-unknown;
}

// cluster PENDING
.marker-cluster-99 {
  background-color: fade(@color-pending, @cluster-inner-transparency);
}

.marker-cluster-99 div {
  background-color: fade(@color-pending, @cluster-outer-transparency);
}

.marker-cluster-99 span {
  color: @color-pending;
}
